// TelemetryTag.h: header file for TTelemetryTag class
//

#pragma once

#include "Telemetry.h"
#include "Engine/Module.h"
#include "Utils/Semaphore.h"
#include "Utils/ssp.h"
#include "Utils/SelectionInfo.h"

//////////////////////////////////////////////////////////////////////
// TTelemetryTag Class
//
enum ETagStates
{
   eNoSet         = 0,
   // Tag states
   eTriggered     = 1<<0, 
   eFormatted     = 1<<1
};

class TELEMETRY_DECL TTelemetryTag : public TModule
{
   // Interface
   public:

   // Constructors and destructor.
   TTelemetryTag(class TTelemetryDevice& device, const CString& ModuleName, HINSTANCE hModInst, bool bAddTag=true);
   virtual ~TTelemetryTag();
   
   // Pure overrides
   virtual CString InputName(int Point) {return Point==2 ? _T("Input") : _T("");};
   virtual CString OutputName(int Point){return Point==2 ? _T("Output") : _T("");};
   virtual bool InputActive(int Point) {return (Point == 2);};
   virtual bool OutputActive(int Point) {return (Point == 2);};
   virtual int InputCount() {return 1;};
   virtual int OutputCount() {return 1;};
   virtual TModule* CreateAnother()   {return static_cast<TModule *>(new TTelemetryTag(m_device, Name, ModuleInstance));};
   virtual TModule* CreateTemporary() {return static_cast<TModule *>(new TTelemetryTag(m_device, Name, ModuleInstance, false));};
   virtual void OnKickBack(CSensorData& sd);

   // Overrides
   virtual int GetModuleType() {return TModule::TELEMETRY;};
   virtual bool ShowProperties(CWnd* pParent);
   virtual void Initialize();
   virtual void Execute(TTime tScheduled, bool bLiveReading);

   // Methods to access buffer and properties.
   CString GetLabel() { return propLabel.AsCString(); };
   int GetNumVals() { return propNumVals.AsInteger(); };
   int GetSequence() { return propSequence.AsInteger(); };
   ETxContent GetDataContent(const CString& strContent);
   ETxContent GetDataContent();
   bool IsEnabled() {return GetDataContent() != eExclude && TModule::IsEnabled();}
   bool GetData(int iLogLine, CSensorData& sd, TTime tRef = TTime::GetCurrentTime());
   bool GetData(int iLogLine, CSensorData& sd, int& iRightDigits, TTime tRef = TTime::GetCurrentTime());
   bool GetDataAtTimestampFromBuffer(TTime t, CSensorData& sd);
   bool GetDataAtIndexFromBuffer(int i, CSensorData& sd, TTime tRef = TTime::GetCurrentTime());
   TTime GetTimeStamp(int i, TTime tRef = TTime::GetCurrentTime());
   int GetAgeInMinutes(int i, TTime tRef = TTime::GetCurrentTime());
   CSensorData GetAlarmData();

   // Methods to lock/unlock the buffer while traversing it.
   bool Lock(DWORD dwWait = INFINITE) {return m_sem.Lock(dwWait);};
   void Unlock() {m_sem.Unlock();};

   // Properties
   TProperty propLabel;
   TProperty propTxContent;
   TProperty propTime;
   TProperty propInterval;
   TProperty propNumVals;
   TProperty propSequence;
   TProperty propRightDigits;
   TProperty propAutoFindLog;
   TProperty propLogFile;
   TProperty propSensorName;

   // Bits used as flags to signal various conditions, to simplify management of alarm 
   // transmissions, etc. 
   int iFlags;

   // Used to make a copy of another tag's properties
   virtual void CopyProperties(TTelemetryTag& tag);
   
   // Compute num vals based on current settings
   bool UpdateNumValues(int& iNumVals, TTimeSpan tsTagInterval, ETxContent content);

   // Implementation
   protected:

   // Comput/update num values based on current settings. Returns true if num vals is set.
   bool UpdateNumValues();

   // Helper function for Execute and OnKickBack
   void HandleAlertFlags(CSensorData& sd, bool& bInAlarm, bool& bOutAlarm, bool& bAlert);

   // Here's where we store incoming data that matches filter criteria. Use Lock() 
   // and Unlock() to synchronize access to the list.
   TArray<CSensorData> m_sd;
   TSemaphore m_sem;

   // Remember the value and state of the data that caused the alarm
   CSensorData m_sdAlarmData;

   // Reference to the device to which this tag belongs.
   class TTelemetryDevice& m_device;
};

//////////////////////////////////////////////////////////////////////
// TTelemetryTags Class
//
class TTypedPtrTag : public CTypedPtrArray<CObArray, TTelemetryTag*>
{
   // This class needed to successfully export TTelemetryTags in ARM build.
};

class TELEMETRY_DECL TTelemetryTags : public TTypedPtrTag
{
public:
   int FindTag(const CString& strLabel);
   void SortTags();
};

extern "C" TSelectionInfo TELEMETRY_DECL g_siDataContent;
